unit FileUtility;

interface
uses SysUtils,Dialogs,Forms,windows,Controls,Graphics,ShellAPI;

Type

  Fileptr=^FileRec;
  FileRec=packed record
    Fn:String;
    Checked:boolean;
    Delete:boolean;
    prior:Fileptr;
    next:Fileptr;
  end;

  PrjPtr=^PrjRec;
  PrjRec=packed record
    PrjName:string;
    prjTgtDir:string;
    prjBackUpType:byte;
    prjBackUpDay:byte;
    prjBackUPTime:byte;
    blRun:byte; //0: no  1:yes Auto BackUp
    prior:PrjPtr;
    next:PrjPtr;
  end;

  procedure Add_FileRec(var Head,Tail:Fileptr; rec:FileRec);
  procedure Release_FileRec(var Head,Tail:Fileptr);
  procedure Delete_FileRec(var head,tail:Fileptr; delhptr:Fileptr);
  procedure Add_PrjRec(var Head,Tail:Prjptr; rec:PrjRec);
  procedure Release_PrjRec(var Head,Tail:Prjptr);
  procedure Delete_PrjRec(var head,tail:Prjptr; delhptr:Prjptr);
  procedure CopyFiles(frmFn,toFn:string);
  procedure MakeWindowOpaque(Form: TForm);
  procedure MakeWindowTransparent(Form: TForm);
  procedure MakeRoundedWindow(Form: TForm; Radius:integer);
  function RecycleFile(InFile: String): Boolean;
  function GetPrjptr(head,tail:Prjptr; Id:integer):Prjptr;
  function GetPrjNum(head,tail:Prjptr):integer;
  function GetFptr(head,tail:Fileptr; Id:integer):Fileptr;

implementation

function GetFptr(head,tail:Fileptr; id:integer):Fileptr;
var
  curf:Fileptr;
  i:integer;
begin
  if Id<0 then begin result:=nil; exit; end;

  i:=0;
  curf:=head;
  while curf<>nil do
  begin
    if i=Id then
    begin
      result:=curf;
      break;
    end;
    inc(i);
    curf:=curf^.next;
  end;

  result:=curf;

end;

function GetPrjptr(head,tail:Prjptr; Id:integer):Prjptr;
var
  i:integer;
  curP:Prjptr;
begin
  if Id<0 then begin result:=nil; exit; end;

  i:=0;
  curP:=head;
  while curP<>nil do
  begin
    if i=Id then
    begin
      result:=curP;
      break;
    end;
    inc(i);
    curP:=curP^.next;
  end;

  result:=curP;

end;

function GetPrjNum(head,tail:Prjptr):integer;
var
  curP:Prjptr;
begin
  curP:=head;
  result:=0;
  while curP<>nil do
  begin
    inc(result);
    curP:=curP^.next;
  end;
end;

//Add one record
procedure Add_PrjRec(var Head,Tail:Prjptr; rec:PrjRec);
begin
  IF Head=NIL THEN
  BEGIN
    NEW(Head);
    Head^.PrjName:=rec.PrjName;
    Head^.PrjTgtDir:=rec.PrjTgtDir;
    Head^.blRun:=rec.blRun;
    Head^.prjBackUpType:=rec.prjBackUpType;
    Head^.prjBackUpDay:=rec.prjBackUpDay;
    Head^.prjBAckUPTime:=rec.prjBAckUpTime;
    Head^.prior:=nil;
    Head^.next:=NIL;
    TAIL:=HEAD;
  END ELSE
  BEGIN
    NEW(TAIL^.NEXT);
    Tail^.next^.prior:=Tail;
    IF HEAD=TAIL THEN HEAD^.NEXT:=TAIL^.NEXT;
    TAIL:=TAIL^.NEXT;
    TAIL^.PrjName:=rec.PrjName;
    TAIL^.PrjTgtDir:=rec.PrjTgtDir;
    TAIL^.blRun:=rec.blRun;
    TAIL^.prjBackUpType:=rec.prjBackUpType;
    TAIL^.prjBackUpDay:=rec.prjBackUpDay;
    TAIL^.prjBAckUPTime:=rec.prjBAckUpTime;
    TAIL^.NEXT:=NIL
  END;
end;


//release all record
procedure Release_PrjRec(var Head,Tail:Prjptr);
var
  next:Prjptr;
BEGIN
  IF head=NIL THEN exit;
  WHILE head<>NIL DO
  BEGIN
    next:=head^.next;
    dispose(head);
    head:=next;
  END;
  head:=nil;
  tail:=nil;
END;

//Delete one record
procedure Delete_PrjRec(var head,tail:Prjptr; delhptr:Prjptr);
begin
  if (head=nil) or (tail=nil) then exit;

  if (head=tail) and (delhptr=head) then
  begin
    dispose(head);
    head:=nil;
    tail:=nil;
    exit;
  end;

  if delhptr=head then
  begin
    head:=head^.next;
    dispose(head^.Prior);
    head^.prior:=nil;
  end
  else if delhptr=tail then
  begin
    tail:=tail^.Prior;
    dispose(tail^.next);
    tail^.next:=nil;
  end
  else
  begin
    delhptr^.Prior^.next:=delhptr^.next;
    delhptr^.next^.Prior:=delhptr^.prior;
    dispose(delhptr);
  end;
end;



//Add one record
procedure Add_FileRec(var Head,Tail:Fileptr; rec:FileRec);
begin
  IF Head=NIL THEN
  BEGIN
    NEW(Head);
    Head^.Fn:=rec.Fn;
    Head^.Checked:=rec.Checked;
    Head^.prior:=nil;
    Head^.next:=NIL;
    TAIL:=HEAD;
  END ELSE
  BEGIN
    NEW(TAIL^.NEXT);
    Tail^.next^.prior:=Tail;
    IF HEAD=TAIL THEN HEAD^.NEXT:=TAIL^.NEXT;
    TAIL:=TAIL^.NEXT;
    TAIL^.Fn:=rec.Fn;
    TAIL^.Checked:=rec.Checked;
    TAIL^.NEXT:=NIL
  END;
end;


//release all record
procedure Release_FileRec(var Head,Tail:Fileptr);
var
  next:Fileptr;
BEGIN
  IF head=NIL THEN exit;
  WHILE head<>NIL DO
  BEGIN
    next:=head^.next;
    dispose(head);
    head:=next;
  END;
  head:=nil;
  tail:=nil;
END;

//Delete one record
procedure Delete_FileRec(var head,tail:Fileptr; delhptr:Fileptr);
begin
  if (head=nil) or (tail=nil) then exit;

  if (head=tail) and (delhptr=head) then
  begin
    dispose(head);
    head:=nil;
    tail:=nil;
    exit;
  end;

  if delhptr=head then
  begin
    head:=head^.next;
    dispose(head^.Prior);
    head^.prior:=nil;
  end
  else if delhptr=tail then
  begin
    tail:=tail^.Prior;
    dispose(tail^.next);
    tail^.next:=nil;
  end
  else
  begin
    delhptr^.Prior^.next:=delhptr^.next;
    delhptr^.next^.Prior:=delhptr^.prior;
    dispose(delhptr);
  end;
end;


//ɮ
procedure CopyFiles(frmFn,toFn:string);
var
  inf,outF,inf1,outf1:File;
  buf,buf1:array[1..2048] of char;
  i,FSize,TotalReaded,Tempreaded,TempWritten:cardinal; 
begin
  IF NOT FileExists(frmFn) then
  begin
    messagedlg(format('%s sb',[ExtractFileName(frmFn)]), mtWarning,[mbOk], 0);
    exit;
  end;

  assignfile(inf,frmFn);
  Reset(inf,1);

  assignfile(outf,toFn);
  rewrite(outf,1);

  Fsize:=filesize(inF);
  totalreaded:=0;
  tempreaded:=0;

  repeat
    blockread(inF,Buf,sizeof(Buf),TempReaded);
    TotalReaded:=TotalReaded+TempReaded;
    blockwrite(Outf,Buf,Tempreaded,TempWritten);
  until EOF(inf) or (TempReaded = 0) or (TempWritten <> TempReaded);

  Close(InF); Close(OutF);

end;


//syz
procedure MakeWindowOpaque(Form: TForm);
begin
  SetWindowRgn(Form.Handle, 0, true);
end;

//syz
procedure MakeWindowTransparent(Form: TForm);
var
  r, r1, r2: HRGN;
  w, h, b, t: integer;
  x0, y0, x1, y1: integer;
  i: integer;
begin
  with Form do
  begin
    w:=Width;
    h:= Height;
    b:= (w - ClientWidth) div 2;
    t:= h - ClientHeight - b;
    x0:= b;
    y0:= t;
    x1:= w - b;
    y1:= h - b;
    r:= CreateRectRgn(0, 0, Screen.Width, Screen.Height);
    r1:= CreateRectRgn(0, 0, w, h);
    r2:= CreateRectRgn(x0, y0, x1, y1);
    CombineRgn(r, r1, r2, RGN_DIFF);
    for i:= 0 to ComponentCount - 1 do
    begin
      if Components[i] is TControl then
      begin
        with Components[i] as TControl do
        begin
          r1:= r; 
          r2:= CreateRectRgn(x0 + Left, y0 + Top, 
                 x0 + Left + Width, y0 + Top + Height);
          CombineRgn(r, r1, r2, RGN_OR);
        end;
      end;
    end;
    SetWindowRgn(Handle, r, true);
    DeleteObject(r);
    DeleteObject(r1);
    DeleteObject(r2);
  end;
end;

//syꨤ
procedure MakeRoundedWindow(Form: TForm; Radius:integer);
var
   myRgn : HRGN;
begin
  //s@ꨤ

  Form.Brush.Style:=bsClear;
  myRgn := Form.Canvas.Handle;
  GetWindowRgn(Form.Handle, myRgn);//oformܽdëwmyRgn
  DeleteObject(myRgn); //RmyRgnWҦ
  myRgn:= CreateroundRectRgn(0,0,Form.Width,Form.Height, Radius, Radius);//إ߶ꨤ|άd
  SetWindowRgn(Form.Handle, myRgn, TRUE); //Nform region]myrgnAó]redrawtrue;

end;

//Nɮײ귽^
function RecycleFile(InFile: String): Boolean;
var
  ShStruct: TSHFileOpStruct;   // Use ShellAPI;
  P1: array [byte] of char;
begin
  Result := False;
  if Not FileExists(InFile) then Exit;
  FillChar(P1,sizeof(P1),0);
  StrPCopy(P1,ExpandFileName(InFile)+#0#0);
  with ShStruct do
  begin
    wnd := 0;
    wFunc := FO_Delete;
    pFrom := P1;
    pTo   := nil;
    fFlags:= FOF_ALLOWUNDO or FOF_NOCONFIRMATION;// and FOF_SIMPLEPROGRESS;
    fAnyOperationsAborted := false;
    hNameMappings := nil;
  end;
  Result := (ShFileOperation(ShStruct) = 0);
  If Not Result then
    Result := FileExists(Infile);
end;

end.
 